#
# notebook.tcl - generic notebook widget
#
# This very module is based on a code taken from book "Effective Tcl/Tk
# Programming" by Mark Harrison and Michael McLennan.  The code has been
# further modified by Toni Ronkko.
#
#  Effective Tcl/Tk Programming
#    Mark Harrison, DSC Communications Corp.
#    Michael McLennan, Bell Labs Innovations for Lucent Technologies
#    Addison-Wesley Professional Computing Series
# 
# ======================================================================
#  Copyright (c) 1996-1997  Lucent Technologies Inc. and Mark Harrison
# ======================================================================
#
#
# Dec 39, 2004  Toni Ronkko <toni at users.playstation2-linux.com>
# 
# $Id: notebook.tcl,v 1.1 2005/01/09 20:59:45 tronkko Exp $
#
# $Log: notebook.tcl,v $
# Revision 1.1  2005/01/09 20:59:45  tronkko
# Initial revision
#
#
#

option add *Notebook.borderWidth 0 widgetDefault
option add *Notebook.relief flat widgetDefault


#
# Create notebook widget.
#
proc notebook_create { win } {
  global nb_info

  frame $win -class Notebook
  pack propagate $win 0
  
  set nb_info($win-count) 0
  set nb_info($win-pages) ""
  set nb_info($win-current) ""
  return $win
}


#
# Insert page to a notebook.  Returns name of frame widget that contains
# the page.  Use the path name to insert components to the page.
#
proc notebook_page { win name } {
  global nb_info
  
  set page "$win.page[incr nb_info($win-count)]"
  lappend nb_info($win-pages) $page
  set nb_info($win-page-$name) $page
  
  frame $page

  # display firstly created page initially
  if { $nb_info($win-count) == 1 } {
    after idle [list notebook_display $win $name]
  }
  return $page
}


#
# Display named page in notebook widget.
#
proc notebook_display { win name } {
  global nb_info

  # get name of frame widget that contains the page
  set page ""
  if { [info exists nb_info($win-page-$name)] } {
    set page $nb_info($win-page-$name)
  } elseif { [winfo exists $win.page$name] } {
    set page $win.page$name
  }
  if { $page == "" } {
    error "bad notebook page \"$name\""
  }

  # Only change pages if not current.  This prevents an ugly flash
  # from happening.
  if { [string compare $nb_info($win-current) $page] != 0 } {
      
    notebook_fix_size $win

    # hide current page for now
    if { $nb_info($win-current) != "" } {
      pack forget $nb_info($win-current)
    }

    # show new page
    pack $page -expand yes -fill both
    set nb_info($win-current) $page
  }
}


#
# Internal, re-size notebook widget to accommodiate larges page.
#
proc notebook_fix_size {win} {
  global nb_info

  update idletasks
  
  set maxw 0
  set maxh 0
  foreach page $nb_info($win-pages) {
    set w [winfo reqwidth $page]
    if { $w > $maxw } {
      set maxw $w
    }
    set h [winfo reqheight $page]
    if { $h > $maxh } {
      set maxh $h
    }
  }
  set bd [$win cget -borderwidth]
  set maxw [expr $maxw+2*$bd]
  set maxh [expr $maxh+2*$bd]
  $win configure -width $maxw -height $maxh
}
