/*
 * pgdemo.c - concept presentation of peach gardens
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 * Nov 18 2004, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: pgdemo.c,v 1.4 2005/06/22 13:24:24 tronkko Exp $
 *
 * $Log: pgdemo.c,v $
 * Revision 1.4  2005/06/22 13:24:24  tronkko
 * method of positioning text has changed
 *
 * Revision 1.3  2005/04/03 16:43:20  tronkko
 * view is required for drawing clouds, use fade()
 *
 * Revision 1.2  2005/01/09 21:01:55  tronkko
 * birds and clouds
 *
 * Revision 1.1  2004/11/20 10:59:26  tronkko
 * Initial revision
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <math.h>

#include "config.h"

#include "graphics.h"
#include "text.h"
#include "image.h"
#include "locate.h"
#include "options.h"
#include "debug.h"
#include "input.h"
#include "ticks.h"
#include "birds.h"
#include "clouds.h"
#include "view.h"
#include "font10x16.h"


/* show screenfull of text */
static void text_screen (const char *buffer);
static void overlay (const char *buffer);
static void print (const char *buffer);

/* demo images */
static struct image overview;
static struct image inside;
static struct image city;
static struct image title;
static struct image chips;
static struct image mapedit;

/* text screens */
static char screen1[] =
"Peach Gardens is a 2d adventure game for Playstation 2 Linux\n"
"kit.  The game is situated on a wonderful piece of land, where\n"
"nothing bad ever happens.  In this world, a young boy is\n"
"about to leave his parents to start a life of his own in some\n"
"far-away land.  As a player, your job is to guide the boy\n"
"through various challenges that shape his skills and prepare\n"
"him for his chosen profession.  The game features exploration,\n"
"puzzles, trading, some role-playing elements and mini-games.\n"
"\n"
"Press X to continue.\n"
;

static char screen2[] =
"The following screens present some images showing what the\n"
"game could look like.  So far the images are just designs\n"
"of what could be done.  There is no game yet.\n"
"\n"
"Press X to continue.\n"
;

static char screen3[] =
"This very screen shows the world map of the game.  This is\n"
"where the player moves between cities and other interesting\n"
"places.  At the beginning of a real game the map would be\n"
"hidden, and only the places visited by the player would\n"
"be visible.\n"
;

static char screen4[] =
"This very screen shows the view of cities, farms and other\n"
"interesting places that can be reached from the world map.\n"
"The player can enter houses and caves to find objects and\n"
"buy stuff.  In a real game there would be people too, and\n"
"the player could talk to them to seek advice or to uncover\n"
"new missions.\n"
;

static char screen5[] =
"This very screen shows the structure of houses and caves.\n"
"The player has various tasks to complete here.  For example,\n"
"the player has to find keys to unlock doors that lead to\n"
"treasure chambers.  Some of the mini-games are also\n"
"triggered inside caves and houses.\n"
;

static char screen6[] =
"Peach Gardens comes with a map editor.  Here, you see the\n"
"main window of the editor.  The left pane shows some of the\n"
"chips available for drawing while the right pane shows a\n"
"map being constructed.  The map editor is implemented in\n"
"Tcl/Tk, and it can be used on a Playstation 2 Linux Kit\n"
"running X11.\n"
;

static char screen7[] =
"This very screen shows an example of the chipset that is\n"
"used for drawing the maps.  The chips are drawn with an\n"
"image editor, such as Gimp, and the map editor is used for\n"
"organizing the chips to create maps and views.  In addition\n"
"to looks, each chip also contains information on terrain in\n"
"that very chip.  Thus, the chips also specify places where\n"
"the player may walk and where a boat is needed.\n"
;

static char screen8[] =
"Peach Gardens is currently under development.  A framework\n"
"using libps2dev has been implemented, and there is a\n"
"prototype of a map editor in Tcl/Tk.\n"
"\n"
"The project is currently seeking game designers and artists\n"
"to contribute to the game.  If you have ever thought about\n"
"creating a game of your own, but have not just found enough\n"
"time to learn programming or write code, this is your change\n"
"to contribute.  Please visit Peach Gardens home page at\n"
"\n"
"    http://playstation2-linux.com/projects/pgardens/\n"
"\n"
"Thank you for your interest.\n"
;


int
main (int argc, char *argv[])
{
  int ev;
  ticks_t start, endtime;
  struct view v;
  
  /* read command line args */
  init_graphics ();
  get_defaults ();
  read_options (&argc, argv);
  
  /* start the demo */
  if (open_screen ()) {
    
    /*** splash screen ***/
    
    /* load title screen */
    init_image (&title, locate("pgardens.pcx"));
    
    clear ();
    bitblt (&title,
        0,0,title.width,title.height,
        0,0,screenwidth(),screenheight());
    swap_buffers ();

    /* load other images */
    init_image (&overview, locate("overview.pcx"));
    init_image (&inside, locate("inside.pcx"));
    init_image (&city, locate("city.pcx"));
    init_image (&chips, locate("chipdemo.pcx"));
    init_image (&mapedit, locate("mapedit.pcx"));
    
    /* load fonts */
    init_font10x16 ();
    setfont (font10x16);

    /* initialize subsystems */
    init_input();
    init_birds();
    init_clouds ();
    init_view (&v);

    /* for bypassing OpenGL bugs */
#if defined(OPENGL)
    clear ();
    text ("OpenGL drivers have bugs so I must print this here and now");
    bitblt (&title,
        0,0,title.width,title.height,
        0,0,screenwidth(),screenheight());
    swap_buffers ();
#endif

    setfg (255, 255, 255);
    start = ticks ();
    endtime = start + 1000;
    while (ticks () < endtime) {
      clear ();
      bitblt (&title,
          0,0,title.width,title.height,
          0,0,screenwidth(),screenheight());
      swap_buffers ();
      ev = get_event ();
      if (ev == SKIP) break;
    }
    
    /*** intro ***/
    setbg (0, 0, 255);
    start = ticks ();
    endtime = start + 30000;
    while (ticks () < endtime) {
      clear ();
      text_screen (screen1);
      swap_buffers ();
      ev = get_event ();
      if (ev == SKIP) break;
    }
    
    /*** help ***/
    start = ticks ();
    endtime = start + 10000;
    while (ticks () < endtime) {
      clear ();
      text_screen (screen2);
      swap_buffers ();
      ev = get_event ();
      if (ev == SKIP) break;
    }
    

    /*** map ***/
    start = ticks ();
    endtime = start + 10000;
    while (ticks () < endtime) {
      clear ();
      bitblt (&overview,
          0,0,overview.width,overview.height,
          0,0,screenwidth(),screenheight());
      draw_clouds (&v);
      animate_clouds ();
      swap_buffers ();
      ev = get_event ();
      if (ev == SKIP) break;
    }
    
    start = ticks ();
    endtime = start + 30000;
    while (ticks () < endtime) {
      clear ();
      bitblt (&overview,
          0,0,overview.width,overview.height,
          0,0,screenwidth(),screenheight());
      draw_clouds (&v);
      animate_clouds ();
      overlay (screen3);
      swap_buffers ();
      ev = get_event ();
      if (ev == SKIP) break;
    }
  

    /*** city ***/
    start = ticks ();
    endtime = start + 10000;
    while (ticks () < endtime) {
      clear ();
      bitblt (&city,
          0,0,city.width,city.height,
          0,0,screenwidth(),screenheight());
      draw_birds (&v);
      animate_birds (&v);
      generate_birds (&v);
      swap_buffers ();
      ev = get_event ();
      if (ev == SKIP) break;
    }
    
    start = ticks ();
    endtime = start + 30000;
    while (ticks () < endtime) {
      clear ();
      bitblt (&city,
          0,0,city.width,city.height,
          0,0,screenwidth(),screenheight());
      draw_birds (&v);
      animate_birds (&v);
      generate_birds (&v);
      overlay (screen4);
      swap_buffers ();
      ev = get_event ();
      if (ev == SKIP) break;
    }
    

    /*** inside view ***/
    start = ticks ();
    endtime = start + 10000;
    while (ticks () < endtime) {
      clear ();
      bitblt (&inside,
          0,0,inside.width,inside.height,
          0,0,screenwidth(),screenheight());
      swap_buffers ();
      ev = get_event ();
      if (ev == SKIP) break;
    }
      
    start = ticks ();
    endtime = start + 30000;
    while (ticks () < endtime) {
      clear ();
      bitblt (&inside,
          0,0,inside.width,inside.height,
          0,0,screenwidth(),screenheight());
      overlay (screen5);
      swap_buffers ();
      ev = get_event ();
      if (ev == SKIP) break;
    }
    
    
    /*** mapeditor ***/
    start = ticks ();
    endtime = start + 10000;
    while (ticks () < endtime) {
      clear ();
      bitblt (&mapedit,
          0,0,mapedit.width,mapedit.height,
          0,0,screenwidth(),screenheight());
      swap_buffers ();
      ev = get_event ();
      if (ev == SKIP) break;
    }
    
    start = ticks ();
    endtime = start + 30000;
    while (ticks () < endtime) {
      clear ();
      bitblt (&mapedit,
          0,0,mapedit.width,mapedit.height,
          0,0,screenwidth(),screenheight());
      overlay (screen6);
      swap_buffers ();
      ev = get_event ();
      if (ev == SKIP) break;
    }
    
    
    /*** chips view ***/
    start = ticks ();
    endtime = start + 10000;
    while (ticks () < endtime) {
      clear ();
      bitblt (&chips,
          0,0,chips.width,chips.height,
          0,0,screenwidth(),screenheight());
      swap_buffers ();
      ev = get_event ();
      if (ev == SKIP) break;
    }
    
    start = ticks ();
    endtime = start + 30000;
    while (ticks () < endtime) {
      clear ();
      bitblt (&chips,
          0,0,chips.width,chips.height,
          0,0,screenwidth(),screenheight());
      overlay (screen7);
      swap_buffers ();
      ev = get_event ();
      if (ev == SKIP) break;
    }
    

    /*** contact ***/
    setbg (0, 0, 255);
    start = ticks ();
    endtime = start + 30000;
    while (ticks () < endtime) {
      clear ();
      text_screen (screen8);
      swap_buffers ();
      ev = get_event ();
      if (ev == SKIP) break;
    }
    
    
    /* deinit */
    done_view (&v);
    done_clouds ();
    done_input ();
    done_birds ();
    close_screen ();
    done_image (&title);
    done_image (&overview);
    done_image (&city);
    done_image (&inside);
    done_image (&chips);
    done_image (&mapedit);
    done_font10x16 ();

    printf ("\n");
    printf ("Remember to visit Peach Gardens home page at:\n");
    printf ("  http://playstation2-linux.com/projects/pgardens/\n");
    printf ("\n");
  } else {
    fprintf (stderr, "cannot switch to graphics mode\n");
  }

  done_graphics ();
  return EXIT_SUCCESS;
}


/*
 * Print multi-line text string to screen.
 */
static void
print(
    const char *buffer)
{
  const char *p;
  char line[200];
  int x = textx();

  p = buffer;
  do{
    int len;
    const char *start = p;

    /* extract line from buffer */
    p = strchr (start, '\n');
    if (p != NULL) {
      len = p - start;
    } else {
      len = strlen (start);
    }
    assert (len < (int) sizeof (line));
    strncpy (&line[0], start, len);
    line[len] = '\0';

    /* print line to screen */
    text (line);

    /* skip to next line */
    settextpos (x, texty() + textheight("M") + 2);
    if (p != NULL) {
      assert (*p == '\n');
      p++;
    }
  } while (p != NULL);
}


/*
 * Show some lines of text at the bottom of the screen.  The background
 * image shows through.
 */
static void
overlay (const char *buffer)
{
  float a;
  int i, ht;
  int lines = 0;
  const char *p = buffer;
  
  /* compute number of lines */
  while (*p != '\0') {
    if (*p == '\n') {
      lines++;
    }
    p++;
  }
    
  /* darken part of image below text */
  ht = lines * (textheight("M") + 2) + textheight("M");
  setfg (0, 0, 80);
  fade (0, screenheight() - ht, screenwidth(), screenheight(), 0xd0);

  /* separator */
  i = screenheight()-ht;
  for (a = 0; a < 3.14; a += 3.14 / 10, --i) {
    int c = ((int)(sin (a) * 0xff)) & 0xff;
    setfg (c, c, c);
    line (0, i, screenwidth(), i);
  }
  
  /* print text with dark gray rim */
  setfg (40, 40, 40);
  settextpos ( 8, 20 + screenheight() - ht); print (buffer);
  settextpos (12, 20 + screenheight() - ht); print (buffer);
  settextpos (10, 18 + screenheight() - ht); print (buffer);
  settextpos (10, 22 + screenheight() - ht); print (buffer);
  setfg (255, 255, 255);
  settextpos (10, 20 + screenheight() - ht); print (buffer);
}


/*
 * Show screenfull of text with large peach gardens logo.
 */
static void
text_screen (const char *buffer)
{
  const int x = 10;
  const int y = 180;
  const char *s = "CONCEPT PRESENTATION";
  
  /* paint background image */
  bitblt (&title,
      0,0,title.width,title.height,
      0,0,screenwidth(),screenheight());

  /* position text just below the logo */
  settextpos (screenwidth() - textwidth(s) - 4, 120);
  setfg (255, 255, 255);  
  text (s);
  
  /* position text just at middle */
  settextpos (x, y);
  
  /* paint screenfull of text */
  setfg (40, 40, 40);
  print (buffer);
}
