/*
 * chapter.c - start new chapters
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 * Apr 11 2005, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: chapter.c,v 1.1 2005/06/22 13:04:37 tronkko Exp $
 *
 * $Log: chapter.c,v $
 * Revision 1.1  2005/06/22 13:04:37  tronkko
 * Initial revision
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "chapter.h"
#include "graphics.h"
#include "input.h"
#include "ticks.h"
#include "font16x24.h"
#include "text.h"
#include "locate.h"
#include "image.h"


/* max length of chapter title string */
#define MAX_CHAPTER_TITLE 64

/* number of ticks to show title */
#define TITLE_DELAY 10000


/*
 * Load image file fn and use it as a background for a chapter title.
 * Chapter title is composed of chapter number (no) and name.
 *
 * BEWARE: following modules must be initialized in order to use the
 * function: font16x24, graphics, input
 */
void
chapter_title(
    int no, const char *fn, const char *title)
{
  ticks_t start, endtime;
  int x, y;
  int ev;
  char chapter[MAX_CHAPTER_TITLE];
  struct image bg;

  assert (no > 0);
  assert (fn != NULL  &&  *fn != '\0');
  assert (title != NULL  &&  *title != '\0');

  /* format title */
  sprintf (chapter, "Chapter %d:", no);
  assert (strlen (chapter) < sizeof (chapter));
  
  /* load background image */
  init_image (&bg, locate (fn));
  
  /* select big font */
  setfont (font16x24);
  
  /* when to exit loop */
  start = ticks ();
  endtime = start + TITLE_DELAY;
  
  /* show text for a while */
  while (ticks () < endtime) {
    clear ();
    bitblt (&bg,
        0,0,bg.width,bg.height,
        0,0,screenwidth(),screenheight());
    
    /* chapter number */
    x = screenwidth() / 2 - textwidth(chapter) / 2;
    y = screenheight() / 2 - textheight("M") - 4;
    settextpos (x, y);
    setfg (0, 0, 40);
    rim (chapter);
    setfg (0xff, 0xff, 0xff);
    text (chapter);    
    
    /* chapter title */
    x = screenwidth() / 2 - textwidth(title) / 2;
    y = screenheight() / 2 + 6;
    settextpos (x, y);
    setfg (0, 0, 40);
    rim (title);
    setfg (0xff, 0xff, 0xff);
    text (title);    
    
    swap_buffers ();
    ev = get_event ();
    if (ev == SKIP) break;
  }
  
  done_image (&bg);
}
