/*
 * chip.h - information on a single chip
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * 
 * Dec 23 2004, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: chip.h,v 1.1 2005/01/09 21:05:14 tronkko Exp $
 *
 * $Log: chip.h,v $
 * Revision 1.1  2005/01/09 21:05:14  tronkko
 * Initial revision
 *
 *
 */
#ifndef CHIP_H
#define CHIP_H

/* terrain types */
enum terrain_t {
  Grass       = 'G',
  Forest      = 'F',
  Road        = 'R', 
  Water       = 'W',
  Mountain    = 'M', 
  Unreachable = 'U'
};


/*
 * Information on a single chip.  In Peach Gardens, the chip entity holds
 * general information on a chip, that is not directly related to chipset or
 * map.  In this context, word "chipset" refers to an object that manages
 * several chips and combines them with graphic images.  Moreover, a chipset
 * object manages indexed collections of chips.  On the other hand, term
 * "map" refers to a 2-dimensional table of chip indices that are resolved
 * with the help of the chipset structure.
 *
 * Note #1: Upper-left coordinate of chip within the chipset image.  The
 * coordinate is expressed as multiples of chip width and height.  All chips
 * are equal in size.
 */
struct chip
{
  int id;                                      /* numerical index of chip */
  int x, y;                                    /* position, see note #1 */
  int z;                                       /* z-coord for overlays */
  int bg;                                      /* background chip (0=off) */
  enum terrain_t terrain[9];                   /* terrain & accessibility */
};


/* convert terrain letter to enumeration */
enum terrain_t parse_terrain (char c);

#endif /*CHIP_H*/
