/*
 * credits.c - credits screen
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 * Dec 28 2004, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: credits.c,v 1.3 2005/06/22 13:09:19 tronkko Exp $
 *
 * $Log: credits.c,v $
 * Revision 1.3  2005/06/22 13:09:19  tronkko
 * added a few photographs
 *
 * Revision 1.2  2005/04/03 16:29:35  tronkko
 * copyright, scale to available screen space
 *
 * Revision 1.1  2005/01/09 21:05:45  tronkko
 * Initial revision
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "credits.h"
#include "resource.h"
#include "graphics.h"
#include "locate.h"
#include "image.h"
#include "text.h"
#include "title.h"
#include "font16x24.h"
#include "scriptfont.h"
#include "font10x16.h"
#include "ticks.h"
#include "input.h"
#include "config.h"


/* switch credit screen every n ticks */
#define CREDITS_DELAY 5000

/* number of tick to show photos */
#define PHOTO_DELAY 5000


/* print text with black rim */
static void ctextxy (int x, int y, const char *s);


/*
 * Show credits.  The function contains an event loop, and the function
 * returns when the user aborts viewing or when all the credits screens have
 * been shown.  Input and title modules must be initialized in order to use
 * the function.
 * 
 * BEWARE: changes current font.
 * BEWARE: script, 10x16 and 16x24 fonts must be loaded in order to show
 * credits.
 */
void
credits (void)
{
  ticks_t start, endtime;
  int x;
  const char *s;
  int ev = NOEVENT;
  float yscale;
  struct image photo;

  /* use large font */
  assert (font16x24 != NULL);
  setfont (font16x24);

  /* scale text position with screen size */
  yscale = screenheight() / 512.0f;

  /* graphics */
  start = ticks ();
  endtime = start + CREDITS_DELAY;
  while (ticks () < endtime) {
    clear ();
    
    /* background image */
    bitblt (title,
        0,0,title->width,title->height,
        0,0,screenwidth(),screenheight());
        
    s = "CREDITS";
    x = screenwidth()-textwidth(s)-8;
    ctextxy (x, 130 * yscale, s);
    
    s = "Characters and tiles by";
    x = screenwidth()/2 - textwidth(s)/2;
    ctextxy (x, 220 * yscale, s);
    
    s = "Reiner ``Tiles'' Prokein";
    x = screenwidth()/2 - textwidth(s)/2;
    ctextxy (x, 300 * yscale, s);
    
    s = "http://www.reinerstileset.de";
    x = screenwidth()/2 - textwidth(s)/2;
    ctextxy (x, 340 * yscale, s);

    swap_buffers ();

    /* exit or skip to next screen */
    ev = get_event ();
    if (ev == SKIP) break;
    if (ev == EXIT) goto exit_credits;
  }
  
  /* credits for title screen */
  start = ticks ();
  endtime = start + CREDITS_DELAY;
  while (ticks () < endtime) {
    clear ();

    /* background image */
    bitblt (title,
        0,0,title->width,title->height,
        0,0,screenwidth(),screenheight());
    
    s = "CREDITS";
    x = screenwidth()-textwidth(s)-8;
    ctextxy (x, 130 * yscale, s);

    s = "Title screen by GAMERMAN4";
    x = screenwidth()/2 - textwidth(s)/2;
    ctextxy (x, 220 * yscale, s);

    s = "Water drop by";
    x = screenwidth()/2 - textwidth(s)/2;
    ctextxy (x, 300 * yscale, s);
    
    s = "greenspoon at Deviant Art";
    x = screenwidth()/2 - textwidth(s)/2;
    ctextxy (x, 340 * yscale, s);
    
    swap_buffers ();
    
    /* exit or skip to next screen */
    ev = get_event ();
    if (ev == SKIP) break;
    if (ev == EXIT) goto exit_credits;    
  }
  
  /* load Jarmo's photograph */
  init_image (&photo, locate ("storm.pcx"));
  
  /* photographers 1 */
  start = ticks ();
  endtime = start + PHOTO_DELAY;
  while (ticks () < endtime) {
    clear ();
    
    /* background image */
    bitblt (&photo,
        0,0,photo.width,photo.height,
        0,0,screenwidth(),screenheight());
    
    s = "Photograph by";
    x = screenwidth()/2 - textwidth(s)/2;
    ctextxy (x, 200 * yscale, s);
    
    s = "Jarmo Ruotsalainen";
    x = screenwidth()/2 - textwidth(s)/2;
    ctextxy (x, 260 * yscale, s);

    s = "http://www.eppunormaali.net";
    x = screenwidth()/2 - textwidth(s)/2;
    ctextxy (x, 300 * yscale, s);
    
    swap_buffers ();
    
    /* exit or skip to next screen */
    ev = get_event ();
    if (ev == SKIP) break;
    if (ev == EXIT) break;
  }

  /* un-load image */
  done_image (&photo);  
  if (ev == EXIT) {
    goto exit_credits;
  }

  /* load Jari's photograph */
  init_image (&photo, locate ("book.pcx"));
  setfont (scriptfont);
  
  /* photographers 2 */
  start = ticks ();
  endtime = start + CREDITS_DELAY;
  while (ticks () < endtime) {
    clear ();
    
    /* background image */
    bitblt (&photo,
        0,0,photo.width,photo.height,
        0,0,screenwidth(),screenheight());

    setfg (0x40, 0x40, 0x40);

    s = "Photograph by";
    x = screenwidth()/2 - textwidth(s)/2 + 40;
    settextpos (x, 160 * yscale);
    text (s);
    
    s = "Jari Tikkanen";
    x = screenwidth()/2 - textwidth(s)/2 + 40;
    settextpos (x, 220 * yscale);
    text (s);
    
    s = "http://www.lapinlahti.net";
    x = screenwidth()/2 - textwidth(s)/2 + 40;
    settextpos (x, 260 * yscale);
    text (s);
    
    s = "Cursive font by";
    x = screenwidth()/2 - textwidth(s)/2 + 40;
    settextpos (x, 320 * yscale);
    text (s);
    
    s = "Robert and Nancy Wall";
    x = screenwidth()/2 - textwidth(s)/2 + 40;
    settextpos (x, 360 * yscale);
    text (s);
    
    swap_buffers ();
    
    /* exit or skip to next screen */
    ev = get_event ();
    if (ev == SKIP) break;
    if (ev == EXIT) break;
  }
  
  /* un-load image */
  setfont (font16x24);
  done_image (&photo);  
  if (ev == EXIT) {
    goto exit_credits;
  }

  /* load Solid's image */
  init_image (&photo, locate ("hitman.pcx"));
  
  start = ticks ();
  endtime = start + PHOTO_DELAY;
  while (ticks () < endtime) {
    clear ();
    
    /* background image */
    bitblt (&photo,
        0,0,photo.width,photo.height,
        0,0,screenwidth(),screenheight());
    
    s = "Battle menu background by";
    x = screenwidth()/2 - textwidth(s)/2;
    ctextxy (x, 220 * yscale, s);
    
    s = "SOLID";
    x = screenwidth()/2 - textwidth(s)/2;
    ctextxy (x, 280 * yscale, s);
    
    swap_buffers ();
    
    /* exit or skip to next screen */
    ev = get_event ();
    if (ev == SKIP) break;
    if (ev == EXIT) break;
  }
  
  /* un-load image */
  done_image (&photo);  
  if (ev == EXIT) {
    goto exit_credits;
  }  
  
  /* credits for story */
  start = ticks ();
  endtime = start + CREDITS_DELAY;
  while (ticks () < endtime) {
    clear ();
    
    /* background image */
    bitblt (title,
        0,0,title->width,title->height,
        0,0,screenwidth(),screenheight());
    
    s = "CREDITS";
    x = screenwidth()-textwidth(s)-8;
    ctextxy (x, 130 * yscale, s);

    s = "Storyline by";
    x = screenwidth()/2 - textwidth(s)/2;
    ctextxy (x, 220 * yscale, s);

    s = "Moosetroop11 and";
    x = screenwidth()/2 - textwidth(s)/2;
    ctextxy (x, 280 * yscale, s);

    s = "Toni Ronkko";
    x = screenwidth()/2 - textwidth(s)/2;
    ctextxy (x, 320 * yscale, s);
    
    swap_buffers ();
    
    /* exit or skip to next screen */
    ev = get_event ();
    if (ev == SKIP) break;
    if (ev == EXIT) goto exit_credits;
  }
  
  /* thanks */
  start = ticks ();
  endtime = start + CREDITS_DELAY;
  while (ticks () < endtime) {
    clear ();
    
    /* background image */
    bitblt (title,
        0,0,title->width,title->height,
        0,0,screenwidth(),screenheight());
    
    setfont (font16x24);
    s = "CREDITS";
    x = screenwidth()-textwidth(s)-8;
    ctextxy (x, 130 * yscale, s);

    setfont (font10x16);
    setfg (60, 60, 60);
    setwrap (20, 220 * yscale, screenwidth() - 20, screenheight());
    settextpos (20, 220 * yscale);
    wrap_text(
        "The authors also gratefully acknowledge the following persons for "
        "their ideas and support:\n"
        "\n"
        "Jari Hyvonen\n");    
    swap_buffers ();
    
    /* exit or skip to next screen */
    ev = get_event ();
    if (ev == SKIP) break;
    if (ev == EXIT) goto exit_credits;
  }

  /* for skipping credits */
exit_credits:

  /*NOP*/;
}


/*
 * Show copyright screen and wait for user actions.  The function returns
 * when user presses a button.
 *
 * BEWARE: the function changes current font.
 */
void
copyright (void)
{
  int ev, x;
  const char *s;
  float yscale;
  
  /* thanks */
  while (1) {
    clear ();
    
    /* scale text position with screen size */
    yscale = screenheight() / 512.0f;
    
    /* background image */
    bitblt (title,
        0,0,title->width,title->height,
        0,0,screenwidth(),screenheight());

    /* title */
    setfont (font16x24);
    s = "COPYING";
    x = screenwidth()-textwidth(s)-8;
    ctextxy (x, 130 * yscale, s);

    /* copyright text */
    setfont (font10x16);
    setfg (60, 60, 60);
    setwrap (10, 165 * yscale, screenwidth() - 5, screenheight());
    settextpos (10, 165 * yscale + textheight ("M"));
    wrap_text(
        "Peach Gardens v" VERSION "\n"
        "Copyright (C) 2005  Toni Ronkko\n"
        "\n"
        "Peach Gardens is free software; you can redistribute it "
        "and/or modify it under the terms of the GNU General Public "
        "License as published by the Free Software Foundation; either "
        "version 2 of the License, or (at your option) any later "
        "version.\n"
        "\n"
        "Peach Gardens is distributed in the hope that it will be "
        "useful, but WITHOUT ANY WARRANTY; without even the implied "
        "warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR "
        "PURPOSE. See the GNU General Public License for more details.\n"
        "\n"
        "You should have received a copy of the GNU General Public "
        "License along with Peach Gardens; if not, write to the Free "
        "Software Foundation, Inc., 59 Temple Place, Suite 330, "
        "Boston, MA 02111-1307 USA");
    
    swap_buffers ();
    
    /* exit or skip to next screen */
    ev = get_event ();
    if (ev == SKIP) break;
    if (ev == EXIT) goto copyright_exit;
  }

  /* for quick exit */
copyright_exit:
  
  /*NOP*/;
}


/*
 * Print string s to (x,y) with black rim around the text.
 */
static void
ctextxy(
    int x, int y, const char *s)
{
  setfg (0, 0, 40);
  settextpos (x-2, y-2); text (s);
  settextpos (x+2, y-2); text (s);
  settextpos (x-3, y  ); text (s);
  settextpos (x+3, y  ); text (s);
  settextpos (x-2, y+2); text (s);
  settextpos (x+2, y+2); text (s);
  settextpos (x,   y-3); text (s);
  settextpos (x,   y+3); text (s);
  setfg (0x94, 0xdc, 0xf4);
  settextpos (x, y); text (s);
}
