/*
 * font.h - bitmap font
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 * Nov 16 2004, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: font.h,v 1.2 2005/06/22 13:06:13 tronkko Exp $
 *
 * $Log: font.h,v $
 * Revision 1.2  2005/06/22 13:06:13  tronkko
 * glyphs are embedded directly to font structure, added callback
 * functions for glyph info and draw
 *
 * Revision 1.1  2005/01/09 21:06:07  tronkko
 * Initial revision
 *
 */
#ifndef FONT_H
#define FONT_H

#include "image.h"


/* number of glyps in a font */
#define MAX_GLYPHS 128


/*
 * A structure containing information on a glyph.  This information is used
 * to compute width of character strings, for example.
 */
struct glyph {
  /* ascii code of glyph */
  int code;
  
  /* position within font bitmap */
  int x, y, w, h;
  
  /* amount of space around character */
  int lead, follow;
  
  /* height above/below baseline */
  int ascent, descent;
};

/* prototype of a font specific function that retrieves info on a glyp */
typedef const struct glyph *getglyph_t (int c);

/* prototype of a function that draws a glyph */
typedef int drawglyph_t (int c);


/*
 * Structure containing information on a font.
 */
struct font
{
  /* image of bitmap fonts */
  struct image data;

  /* character info */
  struct glyph glyphs[MAX_GLYPHS];

  /* callback function to get info on glyph */
  getglyph_t *info;

  /* callback funtion to draw a glyph */
  drawglyph_t *draw;
};


/* index glyphs table */
void init_glyphs (struct glyph *glyphs, const struct glyph *gp, int nglyphs);

/* initialize bitmapped font */
void init_bitmap_font(
    struct font *fptr, const char *fn, const struct glyph *glyphs);

/* initialize generic font */
void init_font (struct font *fptr);

/* de-initialize font */
void done_font (struct font *fptr);

/* set current font used for further text output */
void setfont (struct font *fptr);

/* get pointer to current font */
#define getfont_macro() (cur_font)
struct font *getfont_function (void);

/* use macro version for speed and function for debugging */
#ifdef NDEBUG
# define getfont() getfont_macro()
#else
# define getfont() getfont_function()
#endif

/* retrieve information on a glyph */
const struct glyph *getglyph_function (int c);
#define getglyph_macro(c) (getfont()->info(c))

/* use macro version for speed and function for debugging */
#ifdef NDEBUG
# define getglyph(c) getglyph_macro(c)
#else
# define getglyph(c) getglyph_function(c)
#endif

/* compute width of string s pixels (assuming current font) */
int textwidth (const char *s);

/* get height of current font */
int textheight (const char *s);


/* pointer to current font */
extern struct font *cur_font;

#endif /*FONT_H*/
