/*
 * font16x24.c - 16x24 fixed width font
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 * Apr  3 2005, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: font16x24.c,v 1.1 2005/06/22 13:10:15 tronkko Exp $
 *
 * $Log: font16x24.c,v $
 * Revision 1.1  2005/06/22 13:10:15  tronkko
 * Initial revision
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "font16x24.h"
#include "font.h"
#include "text.h"
#include "locate.h"
#include "graphics.h"


/* publicly usable font structure */
struct font *font16x24 = NULL;
static struct font _font16x24;

/* has module been initialized? */
static int initialized = 0;

/*
 * Character widths etc for 16x24 font.  Note that the measurements in this
 * array are not very precise.  The font started its life as a fixed width
 * font, and most of the metrics do not really describe the individual
 * glyphs very well.
 */
static const struct glyph glyphs[] = {
/*  code   x   y   w  h   spc asc desc*/
  { ' ',   0,  0, 16, 24, 0,0, 20, 4 },
  { '!',  16,  0, 16, 24, 0,0, 20, 4 },
  { '"',  32,  0, 16, 24, 0,0, 20, 4 },
  { '#',  48,  0, 16, 24, 0,0, 20, 4 },
  { '\'',112,  0, 16, 24, 0,0, 20, 4 },
  { '(', 128,  0, 16, 24, 0,0, 20, 4 },
  { ')', 144,  0, 16, 24, 0,0, 20, 4 },
  { '+', 176,  0, 16, 24, 0,0, 20, 4 },
  { ',', 192,  0, 16, 24, 0,0, 20, 4 },
  { '-', 208,  0, 16, 24, 0,0, 20, 4 },
  { '.', 224,  0, 16, 24, 0,0, 20, 4 },
  { '/', 240,  0, 16, 24, 0,0, 20, 4 },

  { '0',   0, 24, 16, 20, 0,0, 20, 4 },
  { '1',  16, 24, 16, 20, 0,0, 20, 4 },
  { '2',  32, 24, 16, 20, 0,0, 20, 4 },
  { '3',  48, 24, 16, 20, 0,0, 20, 4 },
  { '4',  64, 24, 16, 20, 0,0, 20, 4 },
  { '5',  80, 24, 16, 20, 0,0, 20, 4 },
  { '6',  96, 24, 16, 20, 0,0, 20, 4 },
  { '7', 112, 24, 16, 20, 0,0, 20, 4 },
  { '8', 128, 24, 16, 20, 0,0, 20, 4 },
  { '9', 144, 24, 16, 20, 0,0, 20, 4 },
  { ':', 160, 24, 16, 24, 0,0, 20, 4 },
  { ';', 176, 24, 16, 24, 0,0, 20, 4 },
  { '<', 192, 24, 16, 24, 0,0, 20, 4 },
  { '=', 208, 24, 16, 24, 0,0, 20, 4 },
  { '>', 224, 24, 16, 24, 0,0, 20, 4 },
  { '?', 240, 24, 16, 24, 0,0, 20, 4 },

  { 'A',  16, 48, 16, 20, 0,0, 20, 4 },
  { 'B',  32, 48, 16, 20, 0,0, 20, 4 },
  { 'C',  48, 48, 16, 20, 0,0, 20, 4 },
  { 'D',  64, 48, 16, 20, 0,0, 20, 4 },
  { 'E',  80, 48, 16, 20, 0,0, 20, 4 },
  { 'F',  96, 48, 16, 20, 0,0, 20, 4 },
  { 'G', 112, 48, 16, 20, 0,0, 20, 4 },
  { 'H', 128, 48, 16, 20, 0,0, 20, 4 },
  { 'I', 147, 48, 10, 20, 2,2, 20, 4 },
  { 'J', 160, 48, 16, 20, 0,0, 20, 4 },
  { 'K', 176, 48, 16, 20, 0,0, 20, 4 },
  { 'L', 192, 48, 16, 20, 0,0, 20, 4 },
  { 'M', 209, 48, 14, 20, 2,2, 20, 4 },
  { 'N', 224, 48, 16, 20, 0,0, 20, 4 },
  { 'O', 240, 48, 16, 20, 0,0, 20, 4 },

  { 'P',   0, 72, 16, 20, 0,0, 20, 4 },
  { 'Q',  16, 72, 16, 24, 0,0, 20, 4 },
  { 'R',  32, 72, 16, 20, 0,0, 20, 4 },
  { 'S',  48, 72, 16, 20, 0,0, 20, 4 },
  { 'T',  64, 72, 16, 20, 0,0, 20, 4 },
  { 'U',  80, 72, 16, 20, 0,0, 20, 4 },
  { 'V',  96, 72, 16, 20, 0,0, 20, 4 },
  { 'W', 112, 72, 16, 20, 1,1, 20, 4 },
  { 'X', 128, 72, 16, 20, 0,0, 20, 4 },
  { 'Y', 144, 72, 16, 20, 0,0, 20, 4 },
  { 'Z', 160, 72, 16, 20, 0,0, 20, 4 },
  { '[', 176, 72, 16, 24, 0,0, 20, 4 },
  { '\\',192, 72, 16, 24, 0,0, 20, 4 },
  { ']', 208, 72, 16, 24, 0,0, 20, 4 },
  { '^', 224, 72, 16, 24, 0,0, 20, 4 },

  { '`',   0, 96, 16, 24, 0,0, 20, 4 },
  { 'a',  16, 96, 16, 24, 0,0, 20, 4 },
  { 'b',  32, 96, 16, 24, 0,0, 20, 4 },
  { 'c',  48, 96, 16, 24, 0,0, 20, 4 },
  { 'd',  64, 96, 16, 24, 0,0, 20, 4 },
  { 'e',  80, 96, 16, 24, 0,0, 20, 4 },
  { 'f',  96, 96, 16, 24, 0,0, 20, 4 },
  { 'g', 112, 96, 16, 24, 0,0, 20, 4 },
  { 'h', 128, 96, 16, 24, 0,0, 20, 4 },
  { 'i', 147, 96, 10, 20, 2,2, 20, 4 },
  { 'j', 160, 96, 16, 24, 0,0, 20, 4 },
  { 'k', 176, 96, 16, 24, 0,0, 20, 4 },
  { 'l', 196, 96,  8, 20, 2,2, 20, 4 },
  { 'm', 208,104, 15, 12, 1,2, 12, 4 },
  { 'n', 224, 96, 16, 24, 0,0, 20, 4 },
  { 'o', 240, 96, 16, 24, 0,0, 20, 4 },

  { 'p',   0,120, 16, 24, 0,0, 20, 4 },
  { 'q',  16,120, 16, 24, 0,0, 20, 4 },
  { 'r',  32,120, 16, 24, 0,0, 20, 4 },
  { 's',  48,120, 16, 24, 0,0, 20, 4 },
  { 't',  64,120, 16, 24, 0,0, 20, 4 },
  { 'u',  80,120, 16, 24, 0,0, 20, 4 },
  { 'v',  96,120, 16, 24, 0,0, 20, 4 },
  { 'w', 112,120, 16, 24, 0,1, 20, 4 },
  { 'x', 128,120, 16, 24, 0,0, 20, 4 },
  { 'y', 144,120, 16, 24, 0,0, 20, 4 },
  { 'z', 160,120, 16, 24, 0,0, 20, 4 },

  { '\0',  0,  0,  0,  0, 0,0,  0, 0 } 
};  


/*
 * Load 16x24 font to memory.
 */
void
init_font16x24 (void)
{
  assert (!initialized);
  
  /* set up font */
  init_bitmap_font (&_font16x24, locate("16x24.pcx"), glyphs);
  font16x24 = &_font16x24;
  
  initialized = 1;
}


/*
 * Unload font from memory.
 */
void
done_font16x24 (void)
{
  assert (initialized);
  
  assert (font16x24 == &_font16x24);
  done_font (&_font16x24);
  font16x24 = NULL;
  
  initialized = 0;
}


/*
 * Print rim around text in current font color.  This is often used to make
 * some text stand out against colored background.  For example:
 *
 *   setfg(0,0,0);
 *   rim("SOME TEXT");
 *   setfg(255,255,255);
 *   text("SOME TEXT");
 */
void
rim(
    const char *s)
{
  int x = textx ();
  int y = texty ();
  
  setfont (font16x24);
  settextpos (x-2, y-2); text (s);
  settextpos (x+2, y-2); text (s);
  settextpos (x-3, y  ); text (s);
  settextpos (x+3, y  ); text (s);
  settextpos (x-2, y+2); text (s);
  settextpos (x+2, y+2); text (s);
  settextpos (x,   y-3); text (s);
  settextpos (x,   y+3); text (s);
  settextpos (x, y);
}
