/*
 * image.h - I/O image files
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * 
 * Nov  6 2004, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: image.h,v 1.3 2005/04/03 16:30:32 tronkko Exp $
 *
 * $Log: image.h,v $
 * Revision 1.3  2005/04/03 16:30:32  tronkko
 * shadow pixel
 *
 * Revision 1.2  2005/01/09 21:12:01  tronkko
 * find_color
 *
 * Revision 1.1  2004/11/20 10:59:19  tronkko
 * Initial revision
 *
 *
 */
#ifndef IMAGE_H
#define IMAGE_H

/* maximum number of registered images */
#define MAX_IMAGES 100

/* fully transparent pixel color (undefine to disable) */
#define TRANSPARENT_PIXEL 0xff00ff

/* half transparent pixel color for shadows (undefine to disable) */
#define SHADOW_PIXEL 0xff0000


/*
 * RGB color with alpha component. 
 */
struct color
{
  unsigned char r;
  unsigned char g;
  unsigned char b;
  unsigned char a;
};

/*
 * A structure that contains information on an image or a font.
 */
struct image {
  int width;                                   /* size in pixels */
  int height;
  int ncolors;                                 /* # colors */
  struct color palette[256];

  /* private */
  void *data;                                  /* pixel data */  
  FILE *FP;                                    /* input file */
};

/* read image into memory, set fn to NULL to initialize structure */
void init_image (struct image *imp, const char *fn);

/* destroy image from memory */
void done_image (struct image *imp);

/* open image file for reading */
void open_image (struct image *imp, const char *fn);

/* read image data into buffer */
int read_image (struct image *imp, void *buffer, int len);

/* close image file */
void close_image (struct image *imp);

/* register image for drawing */
int register_image(struct image *imp, const char *buffer, int len);

/* remove hardware specific data */
void unregister_image (struct image *imp);

/* find nearest matching color */
struct color *find_color (struct image *imp, int r, int g, int b);

#endif /*IMAGE_H*/
