/*
 * input.h - input devices
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * 
 * Nov 20 2004, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: input.h,v 1.3 2005/06/22 13:06:37 tronkko Exp $
 *
 * $Log: input.h,v $
 * Revision 1.3  2005/06/22 13:06:37  tronkko
 * removed some un-used events
 *
 * Revision 1.2  2005/04/03 16:30:53  tronkko
 * specify joystick device
 *
 * Revision 1.1  2005/01/09 21:12:04  tronkko
 * Initial revision
 *
 *
 */
#ifndef INPUT_H
#define INPUT_H

/* milliseconds before auto-repeating events */
#define TIMEOUT_FIRST  400
#define TIMEOUT_SECOND 120

/* maximum number of events */
#define MAX_EVENTS 13

/* maximum number of event ports */
#define MAX_PORTS 2

/* event numbers (see EVENTBITS if you add something here) */
#define LEFT       0x0
#define RIGHT      0x1
#define UP         0x2
#define DOWN       0x3
#define MENU       0x4                         /* enter game menu */
#define ACTION     0x5                         /* generic do button */
#define EXIT       0x6                         /* exit from something */
#define ERASE      0x7                         /* erase character */
#define PAUSE      0x8                         /* pause game */

/* number of bits to allocate for events */
#define EVENTBITS 4

/* event code meaning that there are no more events */
#define NOEVENT  0xffff

/* event type */
#define PRESS    0x0000
#define RELEASE  0x4000

/* for transforming port bits to an integer */
#define PORTOFFSET 15
#define PORTBITS   1

/* port in which the event occurs */
#define PORT1    (0 << PORTOFFSET)
#define PORT2    (1 << PORTOFFSET)

/* masking event bits */
#define EVENTMASK ((1 << (EVENTBITS))-1)
#define PRESSMASK (PRESS | RELEASE)
#define PORTMASK  (PORT1 | PORT2)

/* compute the index of event in an event table */
#define EVENTID(e)  ((e) & EVENTMASK)

/* event aliases */
#define SKIP ACTION


/* initialize event readers */
void init_input (void);

/* release resources held by the input readers */
void done_input (void);

/* get button/keyboard event */
int get_event (void);

/* is button/key pressed at the moment */
int pressed (int ev);

/* set event to repeat automatically when pressed long enough */
void set_autorepeat (int ev, int flag);

/* init and de-initialize joystick driver */
void init_joy (void);
void done_joy (void);

/* set joystick device (default: /dev/js0) */
void joy_device (const char *dev);

/* get raw hardware specific joystick event */
int joy_event (void);

/* init and de-init keyboard driver */
void init_key (void);
void done_key (void);

/* get raw hardware specific keyboard event */
int key_event (void);

#endif /*INPUT_H*/
