/*
 * mainmenu.c - start new game menu
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 * Apr 11 2005, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: mainmenu.c,v 1.1 2005/06/22 13:10:29 tronkko Exp $
 *
 * $Log: mainmenu.c,v $
 * Revision 1.1  2005/06/22 13:10:29  tronkko
 * Initial revision
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "mainmenu.h"
#include "menu.h"
#include "title.h"
#include "graphics.h"
#include "font16x24.h"
#include "image.h"
#include "text.h"
#include "locate.h"
#include "input.h"


/* paint menu option */
static void paintmenu (struct menu_option *op, int state);


/* menu options */
#define MAIN_OPTIONS 6
static struct menu_option main_options[MAIN_OPTIONS] = {
/*  id         x           y    text */
  { NEW_GAME,  320-8*16/2, 180, "NEW GAME" }, 
  { LOAD_GAME, 320-4*16/2, 220, "LOAD" }, 
  { OPTIONS,   320-7*16/2, 280, "OPTIONS" },
  { COPYRIGHT, 320-7*16/2, 340, "COPYING" },
  { CREDITS,   320-7*16/2, 380, "CREDITS" }, 
  { EXIT_GAME, 320-4*16/2, 420, "EXIT" }
};

#define YESNO_OPTIONS 2
static struct menu_option yesno_options[YESNO_OPTIONS] = {
/*  id         x              y    text */
  { 1,         320-40-3*16/2, 280, "YES" }, 
  { 0,         320+40,        280, "NO" }, 
};

/* menus */
static struct menu main_menu;
static struct menu yesno_menu;


/*
 * Show screen with main menu and return user's selection.
 *
 * BEWARE: following modules must be initialized in order to show main menu:
 * font16x24, title, input, graphics.
 */
int
mainmenu (void)
{
  struct menu_option *op;
  int ev;
  int option;
  const char *title_text = "MAIN MENU";
  int titlex;
  int titley;
  
  /* compute position of title */
  setfont (font16x24);
  titlex = screenwidth()-textwidth(title_text)-8;
  titley = 130 * screenheight() / 512;
  
  init_menu (&main_menu, main_options, MAIN_OPTIONS, &paintmenu);
  while (!menu_finished (&main_menu)) {
    /* paint screen */
    clear ();
    bitblt (title,
        0,0,title->width,title->height,
        0,0,screenwidth(),screenheight());
    settextpos (titlex, titley);

    /* paint title with dark rim */
    setfg (0, 0, 40);
    rim (title_text);
    setfg (0x94, 0xdc, 0xf4);
    text (title_text);
    
    draw_menu (&main_menu);
    swap_buffers ();
    
    /* parse events */
    while ((ev = get_event()) != NOEVENT) {
      switch (ev) {
      case UP:
        menu_select (&main_menu, MENU_PREV);
        break;
        
      case DOWN:
        menu_select (&main_menu, MENU_NEXT);
        break;
        
      case ACTION:
        end_menu (&main_menu);
        break;
        
      case EXIT:
        abort_menu (&main_menu);
        break;
      }
    }
    
  }
  
  /* get selection */
  op = menu_selection (&main_menu);
  if (op != NULL) {
    option = op->id;
    
    /* confirm exit from game */
    if (option == EXIT_GAME) {
      if (!confirm_exit ()) {
        /* do not exit afterall */
        option = CANCEL_MENU;
      }
    }          
  } else {
    /* nothing selected */
    option = CANCEL_MENU;
  }
  
  /* de-initialize menu */
  done_menu (&main_menu);
  
  return option;
}


/*
 * Confirm exit from game.  Returns true if user said yes.
 */
int
confirm_exit (void)
{
  struct menu_option *op;
  int ev;
  int option;
  const char *title_text = "Leave Peach Gardens?";
  int titlex;
  int titley;
  struct image hitman;
  
  /* load background image */
  init_image (&hitman, locate ("hitman.pcx"));
  
  /* compute position of title */
  setfont (font16x24);
  titlex = screenwidth() / 2 - textwidth (title_text) / 2;
  titley = screenheight() / 2 - 30;

  /* set up menu */
  init_menu (&yesno_menu, yesno_options, YESNO_OPTIONS, &paintmenu);
  
  /* select "no" option */
  menu_select (&yesno_menu, 0);
  
  while (!menu_finished (&yesno_menu)) {
    /* paint screen */
    clear ();
    bitblt (&hitman,
        0,0,hitman.width,hitman.height,
        0,0,screenwidth(),screenheight());
    settextpos (titlex, titley);

    /* paint title with dark rim */
    setfg (0, 0, 40);
    rim (title_text);
    setfg (0x94, 0xdc, 0xf4);
    text (title_text);
    
    draw_menu (&yesno_menu);
    swap_buffers ();
    
    /* parse events */
    while ((ev = get_event()) != NOEVENT) {
      switch (ev) {
      case LEFT:
        menu_select (&yesno_menu, MENU_PREV);
        break;
        
      case RIGHT:
        menu_select (&yesno_menu, MENU_NEXT);
        break;
        
      case ACTION:
        end_menu (&yesno_menu);
        break;
        
      case EXIT:
        abort_menu (&yesno_menu);
        break;
      }
    }
    
  }
  
  /* get selection */
  op = menu_selection (&yesno_menu);
  if (op != NULL) {
    option = op->id;
  } else {
    /* nothing selected */
    option = 0;
  }

  /* de-initialize menu */
  done_menu (&yesno_menu);
  
  /* unload image */
  done_image (&hitman);  
  return option;
}


/*
 * Paint menu option to screen.  Options will be re-positioned to screen
 * assuming that options are laid out in relation to 640x512 screen
 * resolution.  If screen geometry is different, options will be
 * re-positioned to fit the new geometry.
 */
static void
paintmenu(
    struct menu_option *op, int state)
{
  /* center text on screen */
  setfont (font16x24);
  settextpos(
      op->x * screenwidth() / 640,
      op->y * screenheight() / 512);

  /* paint text */
  switch (state) {
  /* disabled */
  case MENU_INACTIVE:
    break;
    
  /* selected and blinking */
  case MENU_BLINKON:
    setfg (0, 0, 40);
    rim (op->text);
    setfg (0xff, 0xff, 0x00);
    text (op->text);
    block_cursor ();
    break;

  /* normal */
  case MENU_BLINKOFF:
  case MENU_ACTIVE:
    setfg (0, 0, 40);
    rim (op->text);
    setfg (0x94, 0xdc, 0xf4);
    text (op->text);
    break;
    
  default:
    /* unknown menu state */
    abort ();
  }
}


/*
 * Paint block cursor to current text position.  The cursor is adjusted to
 * fit 16x24 font.  The function does not change text position.
 */
void
block_cursor (void)
{
  int x1 = textx() + 6;
  int y1 = texty() - 22;
  int x2 = x1 + 16;
  int y2 = y1 + 24;
  
  setfg (0, 0, 40);
  triangle (x1-3, y1-2, x2+3, y1-2, x1-3, y2+2);
  triangle (x2+3, y1-2, x1-3, y2+2, x2+3, y2+2);
  
  setfg (0xff, 0xff, 0x00);
  triangle (x1, y1, x2, y1, x1, y2);
  triangle (x2, y1, x1, y2, x2, y2);
}
