/*
 * map.h - map creation and drawing
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * 
 * Dec 23 2004, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: map.h,v 1.2 2005/04/03 16:24:52 tronkko Exp $
 *
 * $Log: map.h,v $
 * Revision 1.2  2005/04/03 16:24:52  tronkko
 * move map in relation to view
 *
 * Revision 1.1  2005/01/09 21:06:51  tronkko
 * Initial revision
 *
 *
 */
#ifndef MAP_H
#define MAP_H

struct map_block; /*forward-decl*/
struct chipset; /*forward-decl*/
struct view; /*forward-decl*/

/*
 * Information on a single map.  Note that a map file may contain multiple
 * maps, and this very structure contains information on a single map only.
 * You typically get multiple map objects for reading a single file.
 */
struct map
{
  struct chipset *chp;
  int w, h;
  struct map_block *data;
};


/*
 * Information on a map block.  The primary purpose of the map block is to
 * maintain information on chip in that very map location.  Moreover, the
 * block object contains information on visibility.
 */
struct map_block
{
  unsigned short id : 14;                      /* index of chip */
  unsigned short region  : 1;                  /* covered by a region */
  unsigned short visible : 1;                  /* is the chip visible */
};


/* set visibility of blocks */
void set_map_visibility (struct map *mp, int visible);

/* initialize map */
void init_map(
    struct map *mp, int w, int h, struct chipset *chp);

/* release map and chipset */
void done_map (struct map *mp);
    
/* find block within map */
/* BEWARE: get_block_macro evaluates mp twice! */
#define get_block_macro(mp,x,y) (&(mp)->data[(x)+(y)*(mp)->w])
struct map_block *get_block_function(struct map *mp, int x, int y);

/* use macro version for speed */
#ifdef NDEBUG
# define get_block(mp,x,y) get_block_macro(mp,x,y)
#else /*debug*/
# define get_block(mp,x,y) get_block_function(mp,x,y)
#endif /*debug*/

/* draw visible portion of map to screen */
void draw_map (struct view *vp, struct map *mp);

#endif /*MAP_H*/
