/*
 * menu.h - simple menu system
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 * Mar  9 2005, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: menu.h,v 1.1 2005/04/03 16:24:56 tronkko Exp $
 *
 * $Log: menu.h,v $
 * Revision 1.1  2005/04/03 16:24:56  tronkko
 * Initial revision
 *
 *
 */
#ifndef MENU_H
#define MENU_H

#include "ticks.h"


/* default menu blink interval */
#define MENU_BLINK_NORMAL 1000

/* menu blink interval after option accepted */
#define MENU_BLINK_CONFIRM 100

/* how many ticks to blink the selected option after accept */
#define MENU_CONFIRM_TICKS 500

/* how long to show disabled items after aborting menu */
#define MENU_ABORT_TICKS 500

/* max length of menu option */
#define MAX_MENU 64

/* for menu_select() */
#define MENU_PREV -2
#define MENU_NEXT -1

/* option state in paint_option() */
#define MENU_ACTIVE   0
#define MENU_INACTIVE 1
#define MENU_BLINKON  2
#define MENU_BLINKOFF 3


/* forward decls */
struct menu;
struct menu_option;


/* prototype of callback function responsible for drawing an option */
typedef void paint_option (struct menu_option *op, int state);


/*
 * Top level menu structure that contains list of menu options.  The
 * structure is typically filled by init_menu function.
 *
 * BEWARE: variable selection typically refers to op table.  However, after
 * aborting menu selection, the variable is set to -1.
 */
struct menu {
  int selected;                                /* index of selected option*/
  struct menu_option *op;                      /* array of options */
  int options;                                 /* number of options */
  paint_option *paint;                         /* paint callback */
  int blink_interval;                          /* length of blink */
  int blink_state;                             /* 1=blink on, 0=off */
  ticks_t blink_time;                          /* next blink */
  ticks_t exit_time;                           /* finish after timeout */
  int finished;                                /* 1=exit from menu loop */
};

/*
 * A single option of a menu.  The structure is typically filled by
 * application programmer.  Menu code is a positive integer number that
 * identifies the menu entry after selection.
 */
struct menu_option {
  int id;                                      /* option identifier */
  int x, y;                                    /* position in screen */
  char text[MAX_MENU];                         /* option text */
};


/* initialize menu structure */
void init_menu(
    struct menu *mp, const struct menu_option *op, int options,
    paint_option *paint);
    
/* deinitialize menu */
void done_menu (struct menu *mp);

/* check if menu selection has been made */
int menu_finished (struct menu *mp);

/* paint menu options to screen */
void draw_menu (struct menu *mp);

/* get currently selected menu item */
struct menu_option *menu_selection (struct menu *mp);

/* select given option or move selection back or forward */
void menu_select (struct menu *mp, int option);

/* accept current option and finish menu */
void end_menu (struct menu *mp);

/* exit menu without selecting an option */
void abort_menu (struct menu *mp);


#endif /*MENU_H*/
