/*
 * script.h - conversation scripts
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 * Apr  4 2005, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: script.h,v 1.1 2005/06/22 13:08:10 tronkko Exp $
 *
 * $Log: script.h,v $
 * Revision 1.1  2005/06/22 13:08:10  tronkko
 * Initial revision
 *
 *
 */
#ifndef SCRIPT_H
#define SCRIPT_H

#include "person.h"


/*
 * Type of phrase.
 */
enum phrase_t {
  InvalidPhrase = 0,
  Say = 1, 
  Exit = 2, 
  Goto = 3, 
  IfYes = 4, 
  IfNo = 5,
  Ask = 6
};


/*
 * A single item of conversation.  The structure is used for remarks,
 * questions and control flow in a conversation script.
 */
struct phrase
{
  struct phrase *next;
  enum phrase_t type;
  int label;                                   /* marks jump targets */
  int target;                                  /* IfYes, IfNo, Goto, Exit */
  enum person_t who;                           /* Say, Ask */
  char *what;                                  /* Say, Ask */
};


/*
 * A conversation script with multiple questions, decisions and phrases.
 */
struct script
{
  int no;                                      /* id of conversation */
  struct phrase *phrases;                      /* start of conversation */
  struct phrase *last;                         /* insertion point */
};


/* init/de-initialize module */
void init_script (void);
void done_script (void);

/* allocate/release scripts */
struct script *alloc_script (int no);
void free_script (struct script *sp);

/* locate script by id */
struct script *find_script (int no);

/* check if script script is valid */
int check_script (struct script *sp);

/* store script to module */
void store_script (struct script *sp);

/* find phrase by id */
struct phrase *find_phrase (struct script *sp, int target);

/* insert phrase to script */
void insert_phrase (struct script *sp, struct phrase *pp);

/* insert say phrase */
void insert_say(
    struct script *sp, int curlabel, enum person_t who, const char *what);

/* allocate new piece of conversation */
struct phrase *alloc_phrase (void);

/* release quotation */
void free_phrase (struct phrase *pp);


#endif /*SCRIPT_H*/
