/*
 * view.c - virtual view of a map
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * 
 * Dec 23 2004, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: view.c,v 1.2 2005/04/03 16:27:25 tronkko Exp $
 *
 * $Log: view.c,v $
 * Revision 1.2  2005/04/03 16:27:25  tronkko
 * init_view(), done_view()
 *
 * Revision 1.1  2005/01/09 21:09:23  tronkko
 * Initial revision
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "view.h"
#include "graphics.h"


/*
 * Initialize view with defaults.
 */
void
init_view(
    struct view *vp)
{
  assert (vp != NULL);

  vp->x = 0.0;
  vp->y = 0.0;
  vp->scale = 1.0;
}


/*
 * De-initialize view.
 */
void
done_view(
    struct view *vp)
{
  assert (vp != NULL);
  
  /* fill fields with garbage to spot errors */
#ifndef NDEBUG
  vp->x = 999999;
  vp->y = 999999;
  vp->scale = 0.0;
#endif
}


/*
 * Get X-coordinate of upper left corner of the visible view.  Note that the
 * function is only used when debugging.  Otherwise, viewx_macro() is used
 * for efficiency.
 */
float
viewx_function(
    struct view *vp)
{
  assert (vp != NULL);
  assert (vp->scale > 0);
  return viewx_macro(vp);
}


/*
 * See viewx_function().
 */
float
viewy_function(
    struct view *vp)
{
  assert (vp != NULL);
  assert (vp->scale > 0);
  return viewy_macro(vp);
}


/*
 * Get view scale.
 */
float
scale_function(
    struct view *vp)
{
  assert (vp != NULL);
  assert (vp->scale > 0);
  return scale_macro(vp);
}


/*
 * Move view so that coordinate (x,y) is at the center of the screen.  If
 * the coordinate is at the edge of the map, the view may not change.  If id
 * does, any changes take place immediately.
 */
void
center_view(
    struct view *vp, float x, float y)
{
  assert (vp != NULL);
  assert (vp->scale > 0);  
  vp->x = x - screenwidth()  / 2 / scale(vp);
  vp->y = y - screenheight() / 2 / scale(vp);
}


/*
 * Set view scale.  That is, zoom or unzoom view.  Change of scale
 * takes place immediately.
 */
void
scale_view(
    struct view *vp, float s)
{
  float x, y;
  assert (vp != NULL);
  assert (vp->scale > 0);  
  assert (0.5 <= s  &&  s <= 10);
  
  /* compute center of screen */
  x = viewx(vp) + screenwidth()  / 2 * scale(vp);
  y = viewy(vp) + screenheight() / 2 * scale(vp);

  /* change scale */
  vp->scale = s;

  /* compute offset to upper left corner of display */
  vp->x = x - screenwidth()  / 2 / s;
  vp->y = y - screenheight() / 2 / s;
}
