/*
 * view.h - virtual view of a map
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * 
 * Dec 23 2004, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: view.h,v 1.2 2005/04/03 16:34:42 tronkko Exp $
 *
 * $Log: view.h,v $
 * Revision 1.2  2005/04/03 16:34:42  tronkko
 * support for multiple views
 *
 * Revision 1.1  2005/01/09 21:14:25  tronkko
 * Initial revision
 *
 *
 */
#ifndef VIEW_H
#define VIEW_H

/*
 * Viewable region.
 */
struct view
{
  float x, y;                                  /* upper-left coordinate */  
  float scale;                                 /* zoom factor */
};


/* get view coordinate */
#define viewx_macro(vp) ((vp)->x)
#define viewy_macro(vp) ((vp)->y)
#define scale_macro(vp) ((vp)->scale)
float viewx_function (struct view *vp);
float viewy_function (struct view *vp);
float scale_function (struct view *vp);

/* define viewx() as macro for speed and as function for debugging */
#ifdef NDEBUG
# define viewx(vp) viewx_macro(vp)
# define viewy(vp) viewy_macro(vp)
# define scale(vp) scale_macro(vp)
#else
# define viewx(vp) viewx_function(vp)
# define viewy(vp) viewy_function(vp)
# define scale(vp) scale_function(vp)
#endif

/* initialize view */
void init_view (struct view *vp);
void done_view (struct view *vp);

/* center view around coordinate (x,y) */
void center_view (struct view *vp, float x, float y);

/* zoom view */
void scale_view (struct view *vp, float s);
  
#endif /*VIEW_H*/
