/*
 * t-graphics.cpp - test graphics interfaces
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * 
 * Nov 2004, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: t-graphics.c,v 1.3 2005/04/03 16:35:54 tronkko Exp $
 *
 * $Log: t-graphics.c,v $
 * Revision 1.3  2005/04/03 16:35:54  tronkko
 * fade()
 *
 * Revision 1.2  2005/01/09 21:15:41  tronkko
 * using ticks for counting time
 *
 * Revision 1.1  2004/11/20 10:58:49  tronkko
 * Initial revision
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <math.h>

#include "config.h"

#if HAVE_UNISTD_H
# include <unistd.h>
#endif

#include "graphics.h"
#include "image.h"
#include "options.h"
#include "debug.h"
#include "ticks.h"

#undef NDEBUG
#include <assert.h>



int
main (int argc, char *argv[])
{
  int ok;
  int chatty = 0;
  (void) argv;
  if (argc > 1) chatty = 1;

  /* read command line args */
  init_graphics ();
  get_defaults ();
  read_options (&argc, argv);
  
  if (chatty) {
    /* open graphics screen */
    ok = open_screen ();
    assert (ok);

    /* synchronization */
#if 1
    {
      int i;
      for (i = 0; i < 200; i++) {
        clear ();
        usleep (10000);
        swap_buffers ();
      }
    }
#endif
    
    /* buffer swapping */
#if 1
    {
      int i;
      
      /* show red line */
      clear ();
      setfg (255, 0, 0);
      line (200, 200, 200, 400);
      swap_buffers ();

      /* any drawing to hidden buffer must not be visible */
      for (i = 0; i < 1000000; i++) {
        int x1 = rand() % screenwidth();
        int y1 = rand() % screenheight();
        int x2 = rand() % screenwidth();
        int y2 = rand() % screenheight();
        line (x1, y1, x2, y2);
      }
      clear ();

      /* prepare second image to hidden buffer */
      line (200, 200, 200, 400);
      line (220, 200, 220, 400);
      swap_buffers ();
      sleep (1);

      /* modify the first buffer */
      line (220, 200, 220, 400);
      line (240, 200, 240, 400);
      swap_buffers ();
      sleep (1);
      
      clear ();      
      line (200, 200, 200, 400);
      line (220, 200, 250, 400);
      line (280, 200, 250, 400);
      swap_buffers ();
      sleep (1);
      
      clear ();
      line (200, 200, 230, 400);
      line (260, 200, 230, 400);
      swap_buffers ();

      /* the triangle should not show */
      triangle (0,0, 500,0, 0, 400);
      sleep (1);
      
      clear ();
      line (200, 200, 230, 400);
      line (260, 200, 230, 400);
      line (280, 200, 280, 400);
      swap_buffers ();
      sleep (1);
    }
#endif

    /* paint test image with lines */
#if 1
    {
      clear ();

      setfg (255, 255, 255);
      line (0, 0, 639, 0);
      line (639, 0, 639, 511);
      line (639, 511, 0, 511);
      line (0, 511, 0, 0);

      setfg (0, 255, 255);
      line (0, 0, screenwidth(), screenheight());
      line (0, screenheight(), screenwidth(), 0);
      
      setfg (255, 0, 0);
      line (320, 100, 320, 412);
      line (100, 256, 540, 256);
      
      swap_buffers ();
      sleep (5);
    }
#endif

    /* count refresh frequency  */
#if 1
    {
      int x = 0;
      ticks_t start, now;
      float elapsed = 0;
      int frames = 0;

      /* synchronize to vertical retrace */
      clear ();
      setfg (255, 255, 255);
      swap_buffers ();

      /* what is the time right now? */
      start = ticks ();

      /* repeat the test for 10 seconds */
      do {
        clear ();
        line (x % screenwidth(), 0, x % screenwidth(), screenheight());
        line (0, x % screenheight(), screenwidth(), x % screenheight());
        ++x;
        swap_buffers ();

        /* time elapsed so far */
        now = ticks ();
        elapsed = ((float) now - start) / 1000.0;
        ++frames;
        
      } while (elapsed < 10.0);
      DEBUG(("refresh frequency: %f frames/s", frames / elapsed));
      sleep (1);
    }
#endif

    /* clipping */
#if 1
    {
      float a;
      setfg (255, 255, 255);
      for (a = 0; a < 6.14; a += 0.01) {
        int x1 = 320;
        int y1 = 256;
        float rx = sin(a)*400.0;
        float ry = cos(a)*400.0;
        clear ();
        line (x1 - rx, y1 - ry, x1 + rx, y1 + ry);
        swap_buffers ();
      }
      sleep (1);
    }
#endif

    /* switch between text and graphic modes */
#if 1
    {
      close_screen ();
      printf ("returning to graphics mode shortly...\n");
      sleep (2);
      ok = open_screen ();
      assert (ok);
      clear ();
      setfg (255, 255, 255);
      line (0, 0, screenwidth(), screenheight());
      line (0, screenheight(), screenwidth(), 0);
      swap_buffers ();
      sleep (1);
    }
#endif
    
    /* paint a few test triangles */
#if 1
    {
      clear ();      
      setfg (255, 255, 0);
      triangle (100, 100, 200, 100, 150, 200);
      setfg (255, 0, 0);
      triangle (200, 100, 150, 200, 250, 200);
      setfg (255, 0, 255);
      triangle (250, 200, 200, 100, 300, 100);     
      swap_buffers ();
      sleep (1);
    }
#endif
    
    /* fill screen with lines */
#if 1
    {
      int x, y, i;
      for (i = 0; i < 255; i++) {
        clear ();
        setfg (i, i, 255-i);
        for (y = 0; y < screenheight(); y++) {
          for (x = 0; x < screenwidth(); x += 16) {
            line (x, y, x + 16, y);
          }
        }
        swap_buffers ();
      }
    }
#endif
    
    /* draw some lines to measure speed */
#if 1
    {
      int i, j;
      long lines = 0;
      ticks_t start, now;
      float elapsed;
      
      clear ();

      /* start timing */
      start = ticks ();

      /* draw some lines  */
      for (j = 0; j < 1000; j++) {
        int r  = rand() % 256;
        int g  = rand() % 256;
        int b  = rand() % 256;
        setfg (r, g, b);
        
        for (i = 0; i < 1000; i++) {
          int x1 = rand() % screenwidth();
          int y1 = rand() % screenheight();
          int x2 = rand() % screenwidth();
          int y2 = rand() % screenheight();
          line (x1, y1, x2, y2);
          ++lines;
        }
      }

      /* compute elapsed time */
      now = ticks ();
      elapsed = ((float) now - start) / 1000.0;

      swap_buffers ();
      
      /*
       * 400MHz P-II, software renderer:  32258.1 lines/s
       * 400MHz P-II, Voodoo Banshee:     83333.6 lines/s
       * 400MHz P-II, GeForce4 Ti 4200:  999946.0 lines/s
       */
      printf ("line drawing performance: %g lines/s\n",
          ((float)lines) / elapsed);
    }
#endif
    
    /* fill screen with small triangles to measure efficiency */
#if 1
    {
      int i, x, y;
      const int s = 20;
      long tris = 0;
      ticks_t start, now;
      float elapsed;
      
      clear ();
      
      /* start timing */
      start = ticks ();
      
      for (i = 0; i < 1024; i++) {
        setfg (i/4, i/4, 255-i/4);
        for (y = 0; y < screenheight(); y += s) {
          for (x = -s/2; x < screenwidth(); x += s) {
            triangle (x,y, x+s,y, x+s/2,y+s);
            triangle (x+s,y, x+s/2,y+s, x+s+s/2,y+s);
            tris += 2;
          }
        }
      }
      
      /* compute elapsed time */
      now = ticks ();
      elapsed = ((float) now - start) / 1000.0;

      clear ();
      swap_buffers ();
      
      /*
       * 400MHz P-II, software renderer:    58088.4 triangles/s
       * 400MHz P-II, Voodoo Banshee:      479217.0 triangles/s
       * 400MHz P-II, GeForce4 Ti 4200:   1917030.0 triangles/s
       */
      printf ("triangle drawing performance: %g triangles/s\n",
          ((float)tris) / elapsed);
    }
#endif

    /* max fill rate */
#if 1
    {
      int i;
      struct image town;
      ticks_t start, now;
      float elapsed = 0;
      long nblts = 0;
      
      init_image (&town, "t-gfx-2.pcx");

      /* show image to user */
      clear ();
      bitblt (&town,
          0,0,town.width,town.height,
          0,0,screenwidth(),screenheight());
      swap_buffers ();
      
      /* start timing */
      start = ticks ();

      /* draw full-screen image multiple times */
      do{
        for (i = 0; i < 1000; i++) {
          bitblt (&town,
              0,0,town.width,town.height,
              0,0,screenwidth(),screenheight());
          nblts++;
        }
      
        /* compute elapsed time */
        now = ticks ();
        elapsed = ((float) now - start) / 1000.0;
        
        /* continue timing at least for 15 seconds */
      } while (elapsed < 15.0);

      swap_buffers ();
      
      /*
       * 400MHz P-II, software renderer:  6.6873 Mpixels/s
       * 400MHz P-II, Voodoo Banshee:    10.5703 Mpixels/s
       * 400MHz P-II, GeForce4 Ti 4200:  245.759 Mpixels/s (w/o textures)
       * 400MHz P-II, GeForce4 Ti 4200:  983.035 Mpixels/s (with textures)
       */
      printf ("texture fill performance: %g Mpixels/s\n",
          (nblts*0.327680) / elapsed);

      done_image (&town);
    }
#endif

    /* image enlargement & clipping */
#if 1
    {
      float f;
      struct image town;
      init_image (&town, "t-gfx-2.pcx");

      for (f = 150; f > 1; f = f - 1) {
        clear ();
        bitblt (&town,
            0, 0, town.width, town.height,
            320-f*3.20, 256-f*2.56, 320+f*3.20, 256+f*2.56);
        swap_buffers ();
      }
      for (f = 1; f < 150; f = f + 1) {
        clear ();
        bitblt (&town,
            0, 0, town.width, town.height,
            320-f*3.20, 256-f*2.56, 320+f*3.20, 256+f*2.56);
        swap_buffers ();
      }
      sleep (5);
      
      done_image (&town);
    }
    
#endif

    /* transparency */
#if 1
    {
      int x, y;
      struct image town, gameover;
      init_image (&gameover, "t-gfx-1.pcx");
      init_image (&town, "t-gfx-2.pcx");

      /* construct image from small pieces */
      clear ();
      for (y = 0; y < town.height; y += 10) {
        for (x = 0; x < town.width; x += 10) {
          bitblt (&town,
              x,y,x+10,y+10,
              x*2,y*2,x*2+20,y*2+20);
        }
      }
      swap_buffers ();
      sleep (5);

      clear ();
      bitblt (&town,
          0,0,town.width,town.height,
          0,0,screenwidth(),screenheight());
      bitblt (&gameover,
          0,0,gameover.width,gameover.height,
          0,0,screenwidth(),screenheight());
      swap_buffers ();
      sleep (5);
      clear ();
      bitblt (&town,
          0,0,town.width,town.height,
          0,0,screenwidth(),screenheight());
      swap_buffers ();
      sleep (5);
      
      done_image (&gameover);
      done_image (&town);
    }
#endif

    /* fade */
#if 1
    {
      int i;
      struct image town;
      int x1 = screenwidth() / 2 - 50;
      int x2 = screenwidth() / 2 + 50;
      init_image (&town, "t-gfx-2.pcx");

      for (i = 0; i < 512; i++) {
        /* paint background image */
        clear ();
        setfg (255, 255, 0);
        bitblt (&town,
            0,0,town.width,town.height,
            0,0,screenwidth(),screenheight());

        if (i < 256) {
          /* fade in */
          fade (0, 0, x1, screenheight(), i);
          fade (x2, 0, screenwidth(), screenheight(), 255 - i);
        } else {
          /* fade out */
          fade (0, 0, x1, screenheight(), 511 - i);
          fade (x2, 0, screenwidth(), screenheight(), i - 256);
        }
        swap_buffers ();
      }
      
      done_image (&town);
    }
#endif
    
    /* close graphics screen */
    close_screen ();
  }
  
  done_graphics ();
  return EXIT_SUCCESS;
}

