/*
 * t-image.c - check image reading
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * 
 * Nov  6 2004, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: t-image.c,v 1.3 2005/04/03 16:36:08 tronkko Exp $
 *
 * $Log: t-image.c,v $
 * Revision 1.3  2005/04/03 16:36:08  tronkko
 * changed some colors in test image
 *
 * Revision 1.2  2005/01/09 21:15:55  tronkko
 * find_color
 *
 * Revision 1.1  2004/11/20 10:59:32  tronkko
 * Initial revision
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "image.h"

#undef NDEBUG
#include <assert.h>



int
main (int argc, char *argv[])
{
  int chatty = 0;
  (void) argv;
  if (argc > 1) chatty = 1;

  /* sanity check */
  {
    int ok;
    struct image img;
    char buffer[8][8];

    init_image (&img, NULL);
    open_image (&img, "t-image-1.pcx");
    assert (img.width == 8);
    assert (img.height == 8);

    /* palette */
    assert (img.ncolors >= 6);
    assert (img.palette[0].r == 0xff);
    assert (img.palette[0].g == 0x00);
    assert (img.palette[0].b == 0x08);
    assert (img.palette[0].a == 0xff);
    assert (img.palette[1].r == 0x00);
    assert (img.palette[1].g == 0xff);
    assert (img.palette[1].b == 0x3c);
    assert (img.palette[1].a == 0xff);
    assert (img.palette[2].r == 0x00);
    assert (img.palette[2].g == 0x00);
    assert (img.palette[2].b == 0xff);
    assert (img.palette[2].a == 0xff);
    assert (img.palette[3].r == 0xff);
    assert (img.palette[3].g == 0xff);
    assert (img.palette[3].b == 0xff);
    assert (img.palette[3].a == 0xff);
    assert (img.palette[4].r == 0x00);
    assert (img.palette[4].g == 0x00);
    assert (img.palette[4].b == 0x00);
    assert (img.palette[4].a == 0xff);
    assert (img.palette[5].r == 0x45);
    assert (img.palette[5].g == 0xdc);
    assert (img.palette[5].b == 0xda);
    assert (img.palette[5].a == 0xff);

    ok = read_image (&img, buffer, sizeof (buffer));
    assert (img.ncolors == 6);
    
    /* red diagonal */
    assert (buffer[0][0] == 0);
    assert (buffer[1][1] == 0);
    assert (buffer[2][2] == 0);
    assert (buffer[3][3] == 0);
    assert (buffer[4][4] == 0);
    assert (buffer[5][5] == 0);
    assert (buffer[6][6] == 0);
    assert (buffer[7][7] == 0);

    /* green line at top */
    assert (buffer[0][1] == 1);
    assert (buffer[0][2] == 1);
    assert (buffer[0][3] == 1);
    assert (buffer[0][4] == 1);
    assert (buffer[0][5] == 1);
    assert (buffer[0][6] == 1);
    assert (buffer[0][7] == 1);

    /* black lines */
    assert (buffer[1][7] == 4);
    assert (buffer[2][7] == 4);
    assert (buffer[3][7] == 4);
    assert (buffer[4][7] == 4);
    assert (buffer[5][7] == 4);
    assert (buffer[6][7] == 4);
    assert (buffer[7][1] == 4);
    assert (buffer[7][2] == 4);
    assert (buffer[7][3] == 4);
    assert (buffer[7][4] == 4);
    assert (buffer[7][5] == 4);
    assert (buffer[7][6] == 4);
   
    /* white background */
    assert (buffer[1][2] == 3);
    assert (buffer[1][3] == 3);
    assert (buffer[2][3] == 3);
    assert (buffer[1][4] == 3);
    assert (buffer[2][4] == 3);
    assert (buffer[1][5] == 3);
    assert (buffer[2][5] == 3);
    assert (buffer[1][6] == 3);
    assert (buffer[2][6] == 3);
    assert (buffer[6][1] == 3);
    assert (buffer[6][5] == 3);
    
    /* center */
    assert (buffer[3][4] == 5);
    assert (buffer[4][3] == 5);

    /* blue line at left */
    assert (buffer[1][0] == 2);
    assert (buffer[2][0] == 2);
    assert (buffer[3][0] == 2);
    assert (buffer[4][0] == 2);
    assert (buffer[5][0] == 2);
    assert (buffer[6][0] == 2);
    assert (buffer[7][0] == 2);
    
    close_image (&img);
    done_image (&img);
  }

  /* find colors */
  {
    int ok;
    struct image img;
    char buffer[8][8];
    struct color *cp;    

    /* read image to memory */
    init_image (&img, NULL);
    open_image (&img, "t-image-1.pcx");
    ok = read_image (&img, buffer, sizeof (buffer));
    assert (img.ncolors == 6);
    
    /* exatch match on blue*/
    cp = find_color (&img, 0, 0, 255);
    assert (cp != NULL);
    assert (cp->r ==   0);
    assert (cp->g ==   0);
    assert (cp->b == 255);

    /* exatch match on green */
    cp = find_color (&img, 0, 255, 60);
    assert (cp->r ==   0);
    assert (cp->g == 255);
    assert (cp->b ==  60);
    
    /* inexact match on green */
    cp = find_color (&img, 0, 255, 0);
    assert (cp->r ==   0);
    assert (cp->g == 255);
    assert (cp->b ==  60);

    /* dark gray equals to black */
    cp = find_color (&img, 20, 20, 20);
    assert (cp->r == 0);
    assert (cp->g == 0);
    assert (cp->b == 0);
    
    close_image (&img);
    done_image (&img);
  }

  return EXIT_SUCCESS;
}
