/*
 * t-input.c - check input interfaces
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * 
 * Nov 21 2004, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: t-input.c,v 1.1 2005/01/09 21:16:22 tronkko Exp $
 *
 * $Log: t-input.c,v $
 * Revision 1.1  2005/01/09 21:16:22  tronkko
 * Initial revision
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "config.h"
#include "input.h"
#include "ticks.h"

#ifdef PS2DEV
#include "tty.h"
#endif

#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif


#undef NDEBUG
#include <assert.h>


int
main (int argc, char *argv[])
{
  int chatty = 0;
  (void) argv;
  if (argc > 1) chatty = 1;
  init_input ();
  
  /* conversion to and from event id */
  {
    assert (EVENTID(0x0001) ==  1);
    assert (EVENTID(0x0002) ==  2);
    assert (EVENTID(0x0004) ==  4);
    assert (EVENTID(0x0008) ==  8);

    /* press or release does not affect to event id */
    assert (EVENTID(0x0001 + RELEASE) ==  1);
    assert (EVENTID(0x0002 + RELEASE) ==  2);
    assert (EVENTID(0x0004 + RELEASE) ==  4);
    assert (EVENTID(0x0008 + RELEASE) ==  8);

    /* ignore port */
    assert (EVENTID(0x0001 + PORT2) ==  1);
    assert (EVENTID(0x0002 + PORT2) ==  2);
    assert (EVENTID(0x0004 + PORT2) ==  4);
    assert (EVENTID(0x0008 + PORT2) ==  8);
  }

  /* chatty reader */
  if (chatty) {
    static int safety_counter = 2000;
    
    printf ("reading keys\n");
    while (1) {
      const char *s;
      int ev, port, press;
      ev = get_event ();

      if (ev != NOEVENT) {
        /* extract game port and press status */
        port = (ev & PORTMASK) >> PORTOFFSET;
        press = ((ev & PRESSMASK) == PRESS);
        
        /* print event info */
        printf ("%6d: port=%d, press=%d, event=",  ticks(), port, press);
        switch ((ev & EVENTMASK)) {
        case LEFT:     s = "left";     break;
        case RIGHT:    s = "right";    break;
        case UP:       s = "up";       break;
        case DOWN:     s = "down";     break;
        case MENU:     s = "menu";     break;
        case ACTION:   s = "action";   break;
        case EXIT:     s = "exit";     break;
        case ERASE:    s = "erase";    break;
        case PAUSE:    s = "pause";    break;
        default:       s = "unknown";
        }
        printf ("%s\n", s);
        
        /* print pressed buttons (port1) */
        printf ("pressed: ");
        if (pressed (LEFT))   printf ("left ");
        if (pressed (RIGHT))  printf ("right ");
        if (pressed (UP))     printf ("up ");
        if (pressed (DOWN))   printf ("down ");
        if (pressed (MENU))   printf ("menu ");
        if (pressed (ACTION)) printf ("action ");
        if (pressed (EXIT))   printf ("exit ");
        if (pressed (ERASE))  printf ("erase ");
        if (pressed (PAUSE))  printf ("pause ");
        printf ("\n");
        fflush (stdout);

        /* process release request */
#ifdef PS2DEV
        if (release_request) {
          switch_tty ();
        }
#endif

        /* do not exit just yet */
        safety_counter = 2000;
        
      } else {
        /* no events in queue */
        static int count = 0;
        
        usleep (1000);
        count = (count + 1) % 400;
        if ((count % 100) == 0) {
          printf ("%c\r", "-\\|/"[count/100]);
          fflush (stdout);
        }

        /* decrease safety counter to bail out after period of inactivity */
        safety_counter--;
      }
      
      /* left and right cannot be pressed at the same time */
      assert (pressed (LEFT) == 0  ||  pressed (RIGHT) == 0);
      assert (pressed (UP) == 0  ||  pressed (DOWN) == 0);

      if (safety_counter < 0) {
        printf ("bailing out\n");
        break;
      }
    }
  }
  
  done_input ();
  return EXIT_SUCCESS;
}
