/*
 * t-text.c - check fonts and printing
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 * Nov 16 2004, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: t-text.c,v 1.4 2005/06/22 13:15:57 tronkko Exp $
 *
 * $Log: t-text.c,v $
 * Revision 1.4  2005/06/22 13:15:57  tronkko
 * compiles under Maemo, text positioning method was changed
 *
 * Revision 1.3  2005/04/03 16:36:27  tronkko
 * check automatic line wrap
 *
 * Revision 1.2  2005/01/09 21:16:41  tronkko
 * draw with multiple fonts
 *
 * Revision 1.1  2004/11/20 10:59:35  tronkko
 * Initial revision
 *
 *
 * Note that fonts cannot be touched under Nokia 770 Internet Tablet unless
 * graphics window is open.  This is bacause non-graphical applications
 * crash if they try to initialize gtk which is also essential for
 * initializing fonts.  Hence, even font information cannot be queried
 * without opening graphics screen.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <math.h>

#include "config.h"

#if HAVE_UNISTD_H
# include <unistd.h>
#endif

#include "graphics.h"
#include "text.h"
#include "options.h"
#include "locate.h"
#include "font10x16.h"
#include "font16x24.h"
#include "scriptfont.h"

#undef NDEBUG
#include <assert.h>


int
main (int argc, char *argv[])
{
  int ok;
  int chatty = 0;
  (void) argv;
  if (argc > 1) chatty = 1;

  /* read command line args */
  init_graphics ();
  get_defaults ();
  read_options (&argc, argv);

  /* load fonts */
  init_font10x16 ();
  init_font16x24 ();
  init_scriptfont ();
  
  /* basic text handling */
#ifndef MAEMO
  {
    setfont (font10x16);
    assert (textwidth ("a") == 10);
    assert (textwidth ("aa") == 20);
    assert (textheight ("M") == 16);
    
    setfont (font16x24);
    assert (textwidth ("a") == 16);
    assert (textwidth ("aa") == 32);
    assert (textheight ("M") == 24);
    
    settextpos (101, 120);
    assert (textx () == 101);
    assert (texty () == 120);
  }
#endif

  /* paint with fonts */
  if (chatty) {
    int ht;
    float a;
    ok = open_screen ();
    assert (ok);   

    /* background color for all tests */
    setbg (0, 0, 200);

    /* get default font height */
    ht = textheight("M");

    /* for circumventing bugs in OpenGL drivers... */
    settextpos (-100, -100);
    setfont (font10x16);
    text ("bypassing bugs...");
    clear ();
    swap_buffers ();
    setfont (font16x24);
    text ("bypassing bugs...");
    clear ();
    swap_buffers ();
    setfont (scriptfont);
    text ("bypassing bugs...");
    clear ();
    swap_buffers ();
    setfont (font10x16);
    
    /* test 1: sanity check */
#if 1
    setfont (font10x16);
    clear ();
    settextpos (5, 16);
    setfg (120, 0, 0);
    line (0, 8, 640, 8);
    line (0, 10, 640, 20);
    setfg (255, 255, 255);
    text ("test 1: sanity check");
    
    settextpos (5, texty() + ht);
    text ("the quick brown fox jumped over the lazy dog");
    settextpos (5, texty () + 20);
    text ("THE QUICK BROWN FOX JUMPED OVER THE LAZY DOG");
    
    swap_buffers ();
    sleep (10);
#endif

    /* test 2: continue output */
#if 1
    setfont (font10x16);
    clear ();
    setfg (255, 255, 255);
    settextpos (5, 16);
    text ("test 2: ");
    textn ("con", 3);
    text ("tinue");
    text (" ");
    textn ("output!!!!!!!!!!!!!!!!!!!!!", 6);    
    settextpos (5, texty() + ht);
    setfont (font16x24);
    text ("the quick brown fox jumped over");
    settextpos (5, texty () + 28);
    text ("THE QUICK BROWN FOX JUMPED OVER");
    settextpos (5, texty () + 50);
    text ("the lazy dog");
    settextpos (5, texty () + 28);
    text ("THE LAZY DOG");
    settextpos (5, texty () + 50);
    swap_buffers ();
    sleep (5);
#endif

    /* test 3: multi-line printing */
#if 1
    setfont (font10x16);
    for (a = 0; a < 15; a += 0.01) {
      int x0, y0, x1, y1, x2, y2;
      clear ();

      x0 = 320 + 200*cos(a);
      y0 = 256 + 200*sin(a);
      x1 = 320 + 100*cos(a-0.4);
      y1 = 256 + 100*sin(a-0.4);
      x2 = 320 + 100*cos(a+0.4);
      y2 = 256 + 100*sin(a+0.4);
      
      setfg (0, 100, 255);
      triangle (320, 256, x1, y1, x2, y2);
      triangle (x0, y0, x1, y1, x2, y2);

      setfg (255, 255, 255);
      settextpos (5, 16);
      text ("test 3: multi-line printing");
      settextpos (20, texty() + ht + 20);

      x1 = 20;
      y1 = texty();
      x2 = screenwidth()-20 - (int)(a * 10);
      y2 = screenheight();
      
      line (x1, y1, x1, y2);
      line (x2, y1, x2, y2);
      
      setwrap (x1, y1, x2, x2);
      wrap_text(
          "Peach Gardens is free software; you can redistribute it "
          "and/or modify it under the terms of the GNU General Public "
          "License as published by the Free Software Foundation; either "
          "version 2 of the License, or (at your option) any later "
          "version.\n"
          "\n"
          "Peach Gardens is distributed in the hope that it will be "
          "useful, but WITHOUT ANY WARRANTY; without even the implied "
          "warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR "
          "PURPOSE. See the GNU General Public License for more details.\n"
          "\n"
          "You should have received a copy of the GNU General Public "
          "License along with Peach Gardens; if not, write to the Free "
          "Software Foundation, Inc., 59 Temple Place, Suite 330, "
          "Boston, MA 02111-1307 USA\n"
          "\n"
          "01234567890123456789012345678901234567890123456789012345678901");
      
      swap_buffers ();
    }
    sleep (5);
#endif

    /* test 5: color */
#if 1
    setfont (font10x16);
    clear ();
    settextpos (5, 16);
    setfg (255, 255, 255);
    text ("test 5: color");
    settextpos (5, texty () + ht + 10);
    setfg (255,   0,   0); text ("red ");
    setfg (  0, 255,   0); text ("green ");
    setfg (  0, 255, 255); text ("blue ");
    setfg (255,   0, 255); text ("violet ");
    setfg (  0,   0,   0); text ("black ");
    settextpos (5, texty () + ht);
    
    setfg (255, 255, 0);
    triangle (textx(),texty(), textx()+300,texty(), textx()+300,texty()+ht);
    setfg (120, 120, 120);
    triangle (textx(),texty()+ht, textx(),texty(), textx()+300,texty()+ht);
    setfg (255,   0,   0); text ("red ");
    setfg (  0, 255,   0); text ("green ");
    setfg (  0, 255, 255); text ("blue ");
    setfg (255,   0, 255); text ("violet ");
    setfg (  0,   0,   0); text ("black ");
    settextpos (5, texty () + ht);

    /* ensure that font color is the same as drawing color */
    setfg (127, 0, 0);
    triangle (200, 200, 250, 200, 250, 220);
    settextpos (200, 210);
    text ("RED");

    setfg (120, 120, 255);
    triangle (200, 240, 280, 240, 280, 260);
    settextpos (200, 250);
    text ("BLUE");

    setfg (80, 80, 80);
    triangle (200, 280, 280, 280, 280, 300);
    settextpos (200, 290);
    text ("GRAY");

    setfg (0, 99, 0);
    triangle (100, 200, 150, 200, 150, 220);
    settextpos (100, 210);
    text ("GREEN");
    
    setfg (120, 120, 0);
    triangle (100, 240, 180, 240, 180, 260);
    settextpos (100, 250);
    text ("YELLOW");
    
    setfg (255, 255, 255);
    triangle (100, 280, 180, 280, 180, 300);
    settextpos (100, 290);
    text ("WHITE");

    
    swap_buffers ();
    sleep (5);
#endif

    /* test 6: rim */
#if 1
    setfont (font10x16);
    clear ();
    settextpos (5, 16);
    setfg (255, 255, 255);
    text ("test 6: rim around text");

    setfg (0, 0, 0);
    settextpos (100-2, 100); text ("Peach Gardens font test");
    settextpos (100+2, 100); text ("Peach Gardens font test");
    settextpos (100, 100-2); text ("Peach Gardens font test");
    settextpos (100, 100+2); text ("Peach Gardens font test");
    setfg (255, 255, 0);
    settextpos (100, 100);
    text ("Peach Gardens font test");
    
    swap_buffers ();
    sleep (5);
#endif

    /* test 7: draw with multiple fonts */
#if 1
    clear ();   
    settextpos (5, 16);
    setfont (font10x16);    
    setfg (255, 255, 255);
    text ("test 7: draw with multiple fonts");
    
    settextpos (5, 40);
    text ("the quick brown fox jumped over the lazy dog");
    settextpos (5, texty () + 20);
    text ("THE QUICK BROWN FOX JUMPED OVER THE LAZY DOG");
    settextpos (5, texty () + 28);
    setfont (font16x24);
    text ("the quick brown fox jumped over");
    settextpos (5, texty () + 28);
    text ("THE QUICK BROWN FOX JUMPED OVER");
    settextpos (5, texty () + 50);
    text ("the lazy dog");
    settextpos (5, texty () + 28);
    text ("THE LAZY DOG");
    settextpos (5, texty () + 50);

    setfont (scriptfont);
    text ("the quick brown fox jumped");
    settextpos (5, texty () + 28);
    text ("THE QUICK BROWN FOX JUMPED");
    settextpos (5, texty () + 28);
    text ("over the lazy dog");
    settextpos (5, texty () + 28);
    text ("OVER THE LAZY DOG");
    settextpos (5, texty () + 28);
    text ("abcdefghijklmnopqrstuvwxyz");
    settextpos (5, texty () + 28);
    text ("NOPQRSTUVWXYZ");
    settextpos (5, texty () + 28);
    text ("ABCDEFGHIJeKLM");
    settextpos (5, texty () + 28);
    
    setfont (scriptfont);
    text ("T");
    setfont (font10x16);    
    text ("he ");
    setfont (font16x24);
    setfg(255, 255, 0);
    text ("font");
    setfg (255, 255, 255);
    setfont (font10x16);
    text (" can be changed in the middle.");
    
    swap_buffers ();
    sleep (20);
#endif

    close_screen ();
  }

  /* de-initialize */
  done_scriptfont ();
  done_font10x16 ();
  done_font16x24 ();
  done_graphics ();
  return EXIT_SUCCESS;
}
